@@ -23,6 +23,7 @@
 #include "../../oor_ctrl_device.h"
 #include "../../../iface_list.h"
 #include "../../../lib/oor_log.h"
+#include "../../../net_mgr/net_mgr.h"
 
 /***************************** FUNCTIONS DECLARATION *************************/
 
@@ -185,9 +186,9 @@ tun_control_dp_recv_msg(sock_t *sl)
 int
 tun_control_dp_send_msg(oor_ctrl_t *ctrl, lbuf_t *buff, uconn_t *udp_conn)
 {
-    int ret, sock, dst_afi;
+    int ret, sock;
     ip_addr_t *src_addr, *dst_addr;
-    lisp_addr_t *ctrl_addr;
+    lisp_addr_t *s_addr;
 
     if (lisp_addr_lafi(&udp_conn->ra) != LM_AFI_IP) {
         OOR_LOG(LDBG_2, "tun_control_dp_send_msg: Destination address %s of UDP connection is not a IP. "
@@ -196,15 +197,16 @@ tun_control_dp_send_msg(oor_ctrl_t *ctrl, lbuf_t *buff, uconn_t *udp_conn)
     }
 
     if (lisp_addr_lafi(&udp_conn->la) != LM_AFI_IP) {
-        dst_afi = lisp_addr_ip_afi(&udp_conn->ra);
-        ctrl_addr = tun_control_dp_get_default_ctrl_address(control_dp_tun.control_dp_data, dst_afi);
-        if (!ctrl_addr) {
-            OOR_LOG(LERR, "tun_control_dp_send_msg: No %s control address found, send aborted!",
-                    (dst_afi == AF_INET) ? "IPv4" : "IPv6");
+        /* Get the src address that will use the kernel */
+        s_addr = net_mgr->netm_get_src_addr_to(&udp_conn->ra);
+        if (!s_addr){
+            OOR_LOG(LERR, "tun_control_dp_send_msg: %s not reachable, send aborted!",
+                    lisp_addr_to_char(&udp_conn->ra));
             return(ERR_SOCKET);
         }
         /* Use as local address the default control address */
-        lisp_addr_copy(&udp_conn->la, ctrl_addr);
+        lisp_addr_copy(&udp_conn->la, s_addr);
+        lisp_addr_del(s_addr);
     }
 
     sock = tun_control_dp_get_output_ctrl_sock(
@@ -260,9 +262,25 @@ tun_control_dp_updated_addr(oor_ctrl_t *ctrl, iface_t *iface,lisp_addr_t *old_ad
         lisp_addr_t *new_addr)
 {
     int addr_afi;
+    iface_t *def_iface;
     tun_ctr_dplane_data_t * data;
     data = (tun_ctr_dplane_data_t *)ctrl->control_data_plane->control_dp_data;
 
+    /* If the address has been removed */
+    if (lisp_addr_is_no_addr(new_addr)){
+        if (lisp_addr_ip_afi(old_addr) == AF_INET){
+            def_iface = data->default_ctrl_iface_v4;
+        }else{
+            def_iface = data->default_ctrl_iface_v6;
+        }
+        if (def_iface == iface){
+            OOR_LOG(LDBG_2, "Removed address from default interface. Recalculate new "
+                                "output control interface");
+            tun_control_dp_set_default_ctrl_ifaces(data);
+        }
+        return (GOOD);
+    }
+
     /* If no default control, recalculate it */
     if (iface->status == UP && !lisp_addr_is_no_addr(new_addr)) {
         addr_afi = lisp_addr_ip_afi(new_addr);
