@@ -21,14 +21,15 @@
 #include "../defs.h"
 #include "../lib/cksum.h"
 #include "../lib/oor_log.h"
-#include "../lib/pointers_table.h"
 #include "../lib/prefixes.h"
+#include "../lib/timers_utils.h"
+#include "../lib/util.h"
 
-
-static int ms_recv_map_request(lisp_ms_t *, lbuf_t *, uconn_t *);
-static int ms_recv_map_register(lisp_ms_t *, lbuf_t *, uconn_t *);
+static int ms_recv_enc_ctrl_msg(lisp_ms_t *ms, lbuf_t *msg, void **ecm_hdr, uconn_t *int_uc);
+static int ms_recv_map_request(lisp_ms_t *ms, lbuf_t *buf,  void *ecm_hdr, uconn_t *int_uc, uconn_t *ext_uc);
+static int ms_recv_map_register(lisp_ms_t *, lbuf_t *,void *ecm_hdr, uconn_t *int_uc, uconn_t *ext_uc);
 static int ms_recv_msg(oor_ctrl_dev_t *, lbuf_t *, uconn_t *);
-static inline lisp_ms_t *lisp_ms_cast(oor_ctrl_dev_t *dev);
+
 
 
 static locator_t *
@@ -151,9 +152,8 @@ lsite_entry_start_expiration_timer(lisp_ms_t *ms, lisp_reg_site_t *rsite)
     oor_timer_t *timer;
 
 
-    timer = oor_timer_create(REG_SITE_EXPRY_TIMER);
-    oor_timer_init(timer, ms, lsite_entry_expiration_timer_cb, rsite,
-            NULL, NULL);
+    timer = oor_timer_without_nonce_new(REG_SITE_EXPRY_TIMER, ms, lsite_entry_expiration_timer_cb,
+            rsite, NULL);
     htable_ptrs_timers_add(ptrs_to_timers_ht,rsite, timer);
 
     /* Give a 2s margin before purging the registered site */
@@ -191,11 +191,37 @@ lsite_entry_update_expiration_timer(lisp_ms_t *ms, lisp_reg_site_t *rsite)
 }
 
 static int
-ms_recv_map_request(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
+ms_recv_enc_ctrl_msg(lisp_ms_t *ms, lbuf_t *msg, void **ecm_hdr, uconn_t *int_uc)
+{
+    packet_tuple_t inner_tuple;
+
+    *ecm_hdr = lisp_msg_pull_ecm_hdr(msg);
+    if (ECM_SECURITY_BIT(*ecm_hdr)){
+        switch (lisp_ecm_auth_type(msg)){
+        default:
+            OOR_LOG(LDBG_2, "Not supported ECM auth type %d",lisp_ecm_auth_type(msg));
+            return (BAD);
+        }
+    }
+    if (lisp_msg_parse_int_ip_udp(msg) != GOOD) {
+        return (BAD);
+    }
+    pkt_parse_inner_5_tuple(msg, &inner_tuple);
+    uconn_init(int_uc, inner_tuple.dst_port, inner_tuple.src_port, &inner_tuple.dst_addr,&inner_tuple.src_addr);
+    *ecm_hdr = lbuf_lisp_hdr(msg);
+
+    return (GOOD);
+}
+
+
+static int
+ms_recv_map_request(lisp_ms_t *ms, lbuf_t *buf,  void *ecm_hdr, uconn_t *int_uc, uconn_t *ext_uc)
 {
 
     lisp_addr_t *   seid        = NULL;
     lisp_addr_t *   deid        = NULL;
+    lisp_addr_t *   neg_pref    = NULL;
+    lisp_addr_t *   aux_deid    = NULL;
     mapping_t *     map         = NULL;
     glist_t *       itr_rlocs   = NULL;
     void *          mreq_hdr    = NULL;
@@ -207,6 +233,12 @@ ms_recv_map_request(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
     lisp_site_prefix_t *    site            = NULL;
     lisp_reg_site_t *       rsite           = NULL;
     uint8_t act_flag;
+    uconn_t send_uc;
+
+    if (!ecm_hdr){
+        OOR_LOG(LDBG_1, "Received a not encapsulated Map Request. Discarding!");
+        return(BAD);
+    }
 
     /* local copy of the buf that can be modified */
     b = *buf;
@@ -249,21 +281,23 @@ ms_recv_map_request(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
         /* Static entries will have null site and not null rsite */
         if (!site && !rsite) {
             /* send negative map-reply with TTL 15 min */
+            neg_pref = mdb_get_shortest_negative_prefix(ms->lisp_sites_db, deid);
 
             if (lisp_addr_is_iid(deid)){
                 act_flag = ACT_NO_ACTION;
             }else{
                 act_flag = ACT_NATIVE_FWD;
             }
-            mrep = lisp_msg_neg_mrep_create(deid, 15, act_flag,A_AUTHORITATIVE,
+            mrep = lisp_msg_neg_mrep_create(neg_pref, 15, act_flag,A_AUTHORITATIVE,
                     MREQ_NONCE(mreq_hdr));
             OOR_LOG(LDBG_1,"The requested EID %s doesn't belong to this Map Server",
                     lisp_addr_to_char(deid));
             OOR_LOG(LDBG_2, "%s, EID: %s, NEGATIVE", lisp_msg_hdr_to_char(mrep),
                     lisp_addr_to_char(deid));
-            send_msg(&ms->super, mrep, uc);
+            send_msg(&ms->super, mrep, ext_uc);
             lisp_msg_destroy(mrep);
             lisp_addr_del(deid);
+            lisp_addr_del(neg_pref);
 
             continue;
         }
@@ -271,13 +305,18 @@ ms_recv_map_request(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
         /* Find if the site actually registered */
         if (!rsite) {
             /* send negative map-reply with TTL 1 min */
-            mrep = lisp_msg_neg_mrep_create(deid, 1, ACT_NATIVE_FWD,A_AUTHORITATIVE,
+            if (site->accept_more_specifics == FALSE){
+                aux_deid = site->eid_prefix;
+            }else{
+                aux_deid = deid;
+            }
+            mrep = lisp_msg_neg_mrep_create(aux_deid, 1, ACT_NATIVE_FWD,A_AUTHORITATIVE,
                     MREQ_NONCE(mreq_hdr));
             OOR_LOG(LDBG_1,"The requested EID %s is not registered",
                                 lisp_addr_to_char(deid));
             OOR_LOG(LDBG_2, "%s, EID: %s, NEGATIVE", lisp_msg_hdr_to_char(mrep),
-                    lisp_addr_to_char(deid));
-            send_msg(&ms->super, mrep, uc);
+                    lisp_addr_to_char(aux_deid));
+            send_msg(&ms->super, mrep, ext_uc);
             lisp_msg_destroy(mrep);
             lisp_addr_del(deid);
             continue;
@@ -287,7 +326,7 @@ ms_recv_map_request(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
         /* If site is null, the request is for a static entry */
 
         /* IF *NOT* PROXY REPLY: forward the message to an xTR */
-        if (site != NULL && site->proxy_reply == FALSE) {
+        if (site != NULL && site->proxy_reply == FALSE && rsite->proxy_reply == FALSE) {
             /* FIXME: once locs become one object, send that instead of mapping */
             forward_mreq(ms, buf, map);
             lisp_msg_destroy(mrep);
@@ -309,8 +348,12 @@ ms_recv_map_request(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
         MREP_NONCE(mrep_hdr) = MREQ_NONCE(mreq_hdr);
 
         /* SEND MAP-REPLY */
-        laddr_list_get_addr(itr_rlocs, lisp_addr_ip_afi(&uc->la), &uc->ra);
-        if (send_msg(&ms->super, mrep, uc) != GOOD) {
+
+        if (map_reply_fill_uconn(&ms->super, itr_rlocs, int_uc, ext_uc, &send_uc) != GOOD){
+            OOR_LOG(LDBG_1, "Couldn't send Map Reply, no itr_rlocs reachable");
+            goto err;
+        }
+        if (send_msg(&ms->super, mrep, ext_uc) != GOOD) {
             OOR_LOG(LDBG_1, "Couldn't send Map-Reply!");
         }
         lisp_msg_destroy(mrep);
@@ -331,31 +374,41 @@ err:
 }
 
 static int
-ms_recv_map_register(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
+ms_recv_map_register(lisp_ms_t *ms, lbuf_t *buf, void *ecm_hdr, uconn_t *int_uc, uconn_t *ext_uc)
 {
     lisp_reg_site_t *rsite = NULL, *new_rsite = NULL;
     lisp_site_prefix_t *reg_pref = NULL;
+    lisp_site_id site_id;
+    lisp_xtr_id xtr_id;
     char *key = NULL;
     lisp_addr_t *eid;
-    lbuf_t b;
-    void *hdr = NULL, *mntf_hdr = NULL;
+    lbuf_t b,*mntf = NULL;
+    void *hdr = NULL, *mntf_hdr = NULL, *enc_mntf_hdr, *mreg_auth_hdr, *mnot_auth_hdr, *rtr_auth_hdr;
     int i = 0;
     mapping_t *m = NULL;
     locator_t *probed = NULL;
-    lbuf_t *mntf = NULL;
     lisp_key_type_e keyid = HMAC_SHA_1_96; /* TODO configurable */
     int valid_records = FALSE;
+    ms_rtr_node_t *rtr = NULL;
+    uconn_t *uc;
 
 
     b = *buf;
     hdr = lisp_msg_pull_hdr(&b);
 
+    if(ecm_hdr){ /*New NAT draft version*/
+        uc = ext_uc;
+    }else{
+        uc = int_uc;
+    }
+
     if (MREG_WANT_MAP_NOTIFY(hdr)) {
         mntf = lisp_msg_create(LISP_MAP_NOTIFY);
         lisp_msg_put_empty_auth_record(mntf, keyid);
     }
 
-    lisp_msg_pull_auth_field(&b);
+
+    mreg_auth_hdr = lisp_msg_pull_auth_field(&b);
 
 
     for (i = 0; i < MREG_REC_COUNT(hdr); i++) {
@@ -380,6 +433,7 @@ ms_recv_map_register(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
             OOR_LOG(LDBG_1, "EID %s not in configured lisp-sites DB "
                     "Discarding mapping", lisp_addr_to_char(eid));
             mapping_del(m);
+            m = NULL;
             continue;
         }
 
@@ -387,19 +441,19 @@ ms_recv_map_register(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
 
         /* if first record, lookup the key */
         if (!key) {
-            if (lisp_msg_check_auth_field(buf, reg_pref->key) != GOOD) {
+            if (lisp_msg_check_auth_field(buf, mreg_auth_hdr, reg_pref->key) != GOOD) {
                 OOR_LOG(LDBG_1, "Message validation failed for EID %s with key "
                         "%s. Stopping processing!", lisp_addr_to_char(eid),
                         reg_pref->key);
-                goto bad;
+                goto err;
             }
             OOR_LOG(LDBG_2, "Message validated with key associated to EID %s",
                     lisp_addr_to_char(eid));
             key = reg_pref->key;
         } else if (strncmp(key, reg_pref->key, strlen(key)) !=0 ) {
-            OOR_LOG(LDBG_1, "EID %s part of multi EID Map-Register has different "
+            OOR_LOG(LDBG_1, "EID %s part of multi EID Map-Register with different "
                     "key! Discarding!", lisp_addr_to_char(eid));
-            continue;
+            goto err;
         }
 
 
@@ -411,6 +465,7 @@ ms_recv_map_register(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
                 OOR_LOG(LDBG_1, "EID %s not in configured lisp-sites DB! "
                         "Discarding mapping!", lisp_addr_to_char(eid));
                 mapping_del(m);
+                m = NULL;
                 continue;
             }
         }else if(lisp_addr_cmp(reg_pref->eid_prefix, eid) !=0) {
@@ -418,7 +473,8 @@ ms_recv_map_register(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
                     "specifics not configured! Discarding",
                     lisp_addr_to_char(eid),
                     lisp_addr_to_char(reg_pref->eid_prefix));
-            lisp_addr_del(eid);
+            mapping_del(m);
+            m = NULL;
             continue;
         }
 
@@ -437,7 +493,7 @@ ms_recv_map_register(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
                 }
                 ms_dump_registered_sites(ms, LDBG_3);
             }
-            reg_pref->proxy_reply = MREG_PROXY_REPLY(hdr);
+            rsite->proxy_reply = MREG_PROXY_REPLY(hdr);
             /* update registration timer */
             lsite_entry_update_expiration_timer(ms, rsite);
         } else {
@@ -447,11 +503,11 @@ ms_recv_map_register(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
             mdb_add_entry(ms->reg_sites_db, mapping_eid(m), new_rsite);
             lsite_entry_start_expiration_timer(ms, new_rsite);
 
-            reg_pref->proxy_reply = MREG_PROXY_REPLY(hdr);
+            new_rsite->proxy_reply = MREG_PROXY_REPLY(hdr);
             ms_dump_registered_sites(ms, LDBG_3);
         }
 
-        if (MREG_WANT_MAP_NOTIFY(hdr)) {
+        if (mntf) {
             lisp_msg_put_mapping(mntf, m, NULL);
             valid_records = TRUE;
         }
@@ -459,30 +515,171 @@ ms_recv_map_register(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
         /* if site previously registered, just remove the parsed mapping */
         if (rsite) {
             mapping_del(m);
+            m = NULL;
         }
+    }
+    if (MREG_IBIT(hdr)){
+        lisp_msg_parse_xtr_id_site_id(&b, &xtr_id, &site_id);
+        OOR_LOG(LDBG_1,"  xTR_ID: %s",get_char_from_xTR_ID(&xtr_id));
+        if (mntf) {
+            lisp_msg_put_xtr_id_site_id(mntf, &xtr_id, &site_id);
+        }
+    }
 
+    /* We don't want Map Notify */
+    if (!mntf){
+        return (GOOD);
     }
 
     /* check if key is initialized, otherwise registration failed */
-    if (mntf && key && valid_records) {
-        mntf_hdr = lisp_msg_hdr(mntf);
-        MNTF_NONCE(mntf_hdr) = MREG_NONCE(hdr);
-        lisp_msg_fill_auth_data(mntf, keyid, key);
-        OOR_LOG(LDBG_1, "%s, IP: %s -> %s, UDP: %d -> %d",
-                lisp_msg_hdr_to_char(mntf), lisp_addr_to_char(&uc->la),
-                lisp_addr_to_char(&uc->ra), uc->lp, uc->rp);
-        send_msg(&ms->super, mntf, uc);
+    if (!key || valid_records == FALSE) {
+        goto err;
     }
+
+    mntf_hdr = lisp_msg_hdr(mntf);
+    MNTF_NONCE(mntf_hdr) = MREG_NONCE(hdr);
+    MNTF_I_BIT(mntf_hdr) = MREG_IBIT(hdr);
+    if (!ecm_hdr){
+        MNTF_R_BIT(mntf_hdr) = MREG_RBIT(hdr);
+    }
+    /* Add Map Notify authentication */
+    mnot_auth_hdr = (uint8_t *)mntf_hdr + sizeof(map_notify_hdr_t);
+    lisp_msg_fill_auth_data(mntf, mnot_auth_hdr, keyid, key);
+
+    /* Check if Map Register is from RTR */
+    if (MREG_IBIT(hdr) && ms->def_rtr_set){
+        rtr = shash_lookup (ms->rtrs_table_by_ip, lisp_addr_to_char(&uc->ra));
+        if (!rtr){
+            OOR_LOG(LDBG_1, "Map-Server: Received Map Register from unknown RTR (%S). Discarding message!",
+                    lisp_addr_to_char(&uc->ra));
+            goto err;
+        }
+        /* Different behaviour depending on the NAT implementation of the RTR*/
+
+        if (rtr->passwd){
+            if(ecm_hdr && ECM_RTR_RELAYED_BIT(ecm_hdr)){
+                /** New version of the draft **/
+                /* Add internal IP/UDP header */
+                pkt_push_inner_udp_and_ip(mntf, LISP_CONTROL_PORT, LISP_CONTROL_PORT,
+                        lisp_addr_ip(&int_uc->la),lisp_addr_ip(&int_uc->ra));
+                /* Add authentication data */
+                rtr_auth_hdr = lisp_msg_push_empty_rtr_auth_data(mntf,rtr->key_type);
+                lisp_msg_fill_rtr_auth_data(mntf,rtr_auth_hdr,rtr->key_type, rtr->passwd);
+                /* And encap contl msg header */
+                enc_mntf_hdr = lisp_msg_push_encap_lisp_header(mntf);
+                ECM_RTR_PROCESS_BIT(enc_mntf_hdr) = 1;
+                ECM_SECURITY_BIT(enc_mntf_hdr) = 1;
+            }else{
+                if (MNTF_R_BIT(mntf_hdr)){
+                    rtr_auth_hdr = lisp_msg_put_empty_auth_record(mntf, keyid);
+                    lisp_msg_fill_auth_data(mntf,rtr_auth_hdr,rtr->key_type, rtr->passwd);
+                }
+            }
+        }
+    }
+    OOR_LOG(LDBG_1, "%s, IP: %s -> %s, UDP: %d -> %d",
+            lisp_msg_hdr_to_char(mntf), lisp_addr_to_char(&uc->la),
+            lisp_addr_to_char(&uc->ra), uc->lp, uc->rp);
+    send_msg(&ms->super, mntf, uc);
+
     lisp_msg_destroy(mntf);
 
     return(GOOD);
 err:
+    mapping_del(m);
+    if (mntf){
+        lisp_msg_destroy(mntf);
+    }
     return(BAD);
-    mapping_del(m);
-    lisp_msg_destroy(mntf);
-bad: /* could return different error */
-    mapping_del(m);
-    lisp_msg_destroy(mntf);
+}
+
+static int
+ms_recv_inf_request(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
+{
+    lbuf_t b, *irep_buf;
+    void *hdr, *irep_hdr, *req_auth_hdr, *rep_auth_hdr;
+    lisp_addr_t *eid, priv_addr, *nat_addr = NULL;
+    int ttl;
+    lisp_site_prefix_t *reg_pref;
+    glist_t *rtr_list = NULL;
+    glist_entry_t *rtr_it;
+    ms_rtr_node_t *rtr;
+    uconn_t r_uc;
+
+    b = *buf;
+    hdr = lisp_msg_pull_hdr(&b);
+
+    if (INF_REQ_R_bit(hdr)) {
+        OOR_LOG(LDBG_1, "Map-Server: Received Info Reply message. Discarding!");
+        return (BAD);
+    }
+
+    req_auth_hdr = lisp_msg_pull_auth_field(&b);
+
+    eid = lisp_addr_new();
+    if (lisp_msg_parse_inf_req_eid_ttl(&b, eid, &ttl) != GOOD) {
+        goto err;
+    }
+
+    /* Verify the EID belongs to the MS */
+
+    reg_pref = mdb_lookup_entry(ms->lisp_sites_db, eid);
+    if (!reg_pref) {
+        OOR_LOG(LDBG_1, "EID %s not in configured lisp-sites DB "
+                "Discarding Info Request...", lisp_addr_to_char(eid));
+        goto err;
+    }
+
+    /* Verify authentication of the msg */
+
+    if (lisp_msg_check_auth_field(buf, req_auth_hdr, reg_pref->key) != GOOD) {
+        OOR_LOG(LDBG_1, "Info Request validation failed for EID %s with key "
+                "%s. Stopping processing!", lisp_addr_to_char(eid),
+                reg_pref->key);
+        goto err;
+    }
+
+    /** Generate Info Reply message **/
+
+    /* Generate rtr list */
+    rtr_list = glist_new();
+    glist_for_each_entry(rtr_it,ms->def_rtr_set->rtr_list){
+        rtr = (ms_rtr_node_t *)glist_entry_data(rtr_it);
+        glist_add(rtr->addr,rtr_list);
+    }
+
+    lisp_addr_set_lafi(&priv_addr, LM_AFI_NO_ADDR);
+    nat_addr = lisp_addr_new_init_nat(LISP_CONTROL_PORT, &uc->la,
+            uc->rp,&uc->ra, &priv_addr, rtr_list);
+
+
+    irep_buf = lisp_msg_inf_reply_create(eid, nat_addr,
+            reg_pref->key_type, ms->def_rtr_set->ttl);
+    if (!irep_buf){
+        OOR_LOG(LDBG_1,"ms_recv_inf_request: Can not generate Info Reply message");
+        goto err;
+    }
+
+    glist_destroy(rtr_list);
+    lisp_addr_del(nat_addr);
+    lisp_addr_del(eid);
+
+    irep_hdr = lisp_msg_hdr(irep_buf);
+    INF_REQ_NONCE(irep_hdr) = INF_REQ_NONCE(hdr);
+
+    rep_auth_hdr = irep_hdr + sizeof(info_nat_hdr_t);
+    lisp_msg_fill_auth_data(irep_buf, rep_auth_hdr, reg_pref->key_type, reg_pref->key);
+
+    uconn_init(&r_uc, LISP_CONTROL_PORT, uc->rp, &uc->la, &uc->ra);
+    send_msg(&ms->super, irep_buf, &r_uc);
+
+    lisp_msg_destroy(irep_buf);
+
+    return(GOOD);
+err:
+    glist_destroy(rtr_list);
+    lisp_addr_del(eid);
+    lisp_addr_del(nat_addr);
     return(BAD);
 }
 
@@ -553,7 +750,7 @@ ms_dump_registered_sites(lisp_ms_t *ms, int log_level)
 
 }
 
-static inline lisp_ms_t *
+inline lisp_ms_t *
 lisp_ms_cast(oor_ctrl_dev_t *dev)
 {
     /* make sure */
@@ -567,29 +764,39 @@ ms_recv_msg(oor_ctrl_dev_t *dev, lbuf_t *msg, uconn_t *uc)
     int ret = BAD;
     lisp_msg_type_e type;
     lisp_ms_t *ms;
+    void *ecm_hdr = NULL;
+    uconn_t *int_uc, *ext_uc = NULL, aux_uc;
 
     ms = lisp_ms_cast(dev);
     type = lisp_msg_type(msg);
 
     if (type == LISP_ENCAP_CONTROL_TYPE) {
-        if (lisp_msg_ecm_decap(msg, &uc->rp) != GOOD)
+        if (ms_recv_enc_ctrl_msg(ms, msg, &ecm_hdr, &aux_uc)!=GOOD){
             return (BAD);
+        }
         type = lisp_msg_type(msg);
+        ext_uc = uc;
+        int_uc = &aux_uc;
+        OOR_LOG(LDBG_1, "Map-Server: Received Encapsulated %s", lisp_msg_hdr_to_char(msg));
+    }else{
+        int_uc = uc;
     }
 
      switch(type) {
      case LISP_MAP_REQUEST:
-         ret = ms_recv_map_request(ms, msg, uc);
+         ret = ms_recv_map_request(ms, msg, ecm_hdr, int_uc, ext_uc);
          break;
      case LISP_MAP_REGISTER:
-         ret = ms_recv_map_register(ms, msg, uc);
+         ret = ms_recv_map_register(ms, msg, ecm_hdr, int_uc, ext_uc);
          break;
      case LISP_MAP_REPLY:
      case LISP_MAP_NOTIFY:
-     case LISP_INFO_NAT:
          OOR_LOG(LDBG_3, "Map-Server: Received control message with type %d."
                  " Discarding!", type);
          break;
+     case LISP_INFO_NAT:
+         ret = ms_recv_inf_request(ms, msg, uc);
+         break;
      default:
          OOR_LOG(LDBG_3, "Map-Server: Received unidentified type (%d) control "
                  "message", type);
@@ -647,6 +854,13 @@ ms_ctrl_construct(oor_ctrl_dev_t *dev)
     ms->reg_sites_db = mdb_new();
     ms->lisp_sites_db = mdb_new();
 
+    ms->rtrs_set_table = shash_new_managed((free_value_fn_t)ms_rtr_set_del);
+    // rtrs_table_by_name and rtrs_table_by_ip points to the same pointers value. Only one
+    // should be managed
+    ms->rtrs_table_by_name = shash_new_managed((free_value_fn_t)ms_rtr_node_del);
+    ms->rtrs_table_by_ip = shash_new();
+    ms->def_rtr_set = NULL;
+
     if (!ms->reg_sites_db || !ms->lisp_sites_db) {
         return(BAD);
     }
@@ -662,6 +876,10 @@ ms_ctrl_destruct(oor_ctrl_dev_t *dev)
     lisp_ms_t *ms = lisp_ms_cast(dev);
     mdb_del(ms->lisp_sites_db, (mdb_del_fct)lisp_site_prefix_del);
     mdb_del(ms->reg_sites_db, (mdb_del_fct)lisp_reg_site_del);
+    shash_destroy(ms->rtrs_set_table);
+    shash_destroy(ms->rtrs_table_by_name);
+    shash_destroy(ms->rtrs_table_by_ip);
+    // ms->def_rtr_set is destroyed when destroying ms->rtrs_set_table
 }
 
 void
@@ -680,6 +898,10 @@ ms_ctrl_run(oor_ctrl_dev_t *dev)
     OOR_LOG (LDBG_1, "****** Summary of the configuration ******");
     ms_dump_configured_sites(ms, LDBG_1);
     ms_dump_registered_sites(ms, LDBG_1);
+    if (ms->def_rtr_set){
+        OOR_LOG (LDBG_1, "*** Announced RTR list ***");
+        ms_rtr_set_dump(ms->def_rtr_set, LDBG_1);
+    }
 
     OOR_LOG(LDBG_1, "Starting Map-Server ...");
 }
@@ -697,3 +919,74 @@ ctrl_dev_class_t ms_ctrl_class = {
         .route_update = ms_route_update,
         .get_fwd_entry = ms_get_fwd_entry
 };
+
+
+
+/*****  Basic rtr_node_t and rtr_set_t functions *****/
+
+ms_rtr_node_t *
+ms_rtr_node_new_init(char *id, lisp_addr_t *addr, char *passwd)
+{
+    ms_rtr_node_t *rtr = xzalloc(sizeof(ms_rtr_node_t));
+    if (!rtr){
+        OOR_LOG(LDBG_1, "Can't allocate a new rtr_node_t");
+        return (NULL);
+    }
+    rtr->id = strdup(id);
+    rtr->addr = lisp_addr_clone(addr);
+    rtr->passwd = strdup(passwd);
+    // Message Authentication Code hardcoded
+    rtr->key_type = HMAC_SHA_1_96;
+
+    return(rtr);
+}
+
+void
+ms_rtr_node_del(ms_rtr_node_t * rtr)
+{
+    lisp_addr_del(rtr->addr);
+    free(rtr->id);
+    free(rtr->passwd);
+    free(rtr);
+}
+
+ms_rtr_set_t *
+ms_rtr_set_new_init(char *id, int ttl)
+{
+    ms_rtr_set_t *rtr_set = xzalloc(sizeof(ms_rtr_set_t));
+    if (!rtr_set){
+        OOR_LOG(LDBG_1, "Can't allocate a new rtr_set_t");
+        return (NULL);
+    }
+    rtr_set->id = strdup(id);
+    rtr_set->ttl = ttl;
+    rtr_set->rtr_list = glist_new();
+
+    return (rtr_set);
+}
+
+void
+ms_rtr_set_del(ms_rtr_set_t *rtr_set)
+{
+    free(rtr_set->id);
+    glist_destroy(rtr_set->rtr_list);
+    free(rtr_set);
+}
+
+void
+ms_rtr_set_dump(ms_rtr_set_t *rtr_set, int log_level)
+{
+    glist_entry_t *it;
+    int ctr = 0;
+    ms_rtr_node_t * rtr;
+
+    if (!is_loggable(log_level)){
+        return;
+    }
+    OOR_LOG (log_level, " RTR set name \"%s\" , ttl= %d", rtr_set->id, rtr_set->ttl);
+    glist_for_each_entry(it,rtr_set->rtr_list){
+        ctr++;
+        rtr = (ms_rtr_node_t *)glist_entry_data (it);
+        OOR_LOG(log_level,"    [%d] =>  %s",ctr,lisp_addr_to_char(rtr->addr));
+    }
+}
