@@ -573,12 +573,43 @@ tun_updated_route (int command, iface_t *iface, lisp_addr_t *src_pref,
 int
 tun_updated_addr(iface_t *iface, lisp_addr_t *old_addr, lisp_addr_t *new_addr)
 {
-    int old_addr_lafi, new_addr_ip_afi;
+    int old_addr_lafi,old_addr_ip_afi, new_addr_lafi,new_addr_ip_afi;
     int sckt;
+    iface_t * def_iface;
     tun_dplane_data_t *data;
 
     data = (tun_dplane_data_t *)dplane_tun.datap_data;
     old_addr_lafi = lisp_addr_lafi(old_addr);
+    new_addr_lafi = lisp_addr_lafi(new_addr);
+
+
+
+    /* Process if the address has been removed */
+    if (new_addr_lafi == LM_AFI_NO_ADDR){
+        old_addr_ip_afi = lisp_addr_ip_afi(old_addr);
+        /* Close sockets associated with address */
+        switch (old_addr_ip_afi) {
+        case AF_INET:
+            close (iface->out_socket_v4);
+            iface->out_socket_v4 = 0;
+            def_iface = data->default_out_iface_v4;
+            break;
+        case AF_INET6:
+            close (iface->out_socket_v6);
+            iface->out_socket_v6 = 0;
+            def_iface = data->default_out_iface_v6;
+            break;
+        }
+        if (def_iface == iface){
+            OOR_LOG(LDBG_2, "Removed address from default interface. Recalculate new "
+                    "output interface");
+            tun_set_default_output_ifaces();
+        }
+        del_rule(old_addr_ip_afi, 0, iface->iface_index, iface->iface_index, RTN_UNICAST,
+                        old_addr, NULL, 0);
+        return (GOOD);
+    }
+
     new_addr_ip_afi = lisp_addr_ip_afi(new_addr);
 
     /* Check if the detected change of address is the same. */
@@ -883,7 +914,7 @@ tun_rm_fwd_from_entry(lisp_addr_t *eid_prefix, uint8_t is_local)
     }
 
     if (strcmp(eid_prefix_char,FULL_IPv4_ADDRESS_SPACE) == 0){ // Update of the PeTR list for IPv4 EIDs or RTR list
-        OOR_LOG(LDBG_3, "tun_rm_fwd_from_entry: Removing all the forwading entries association with the PeTRs for IPv4 EIDs");
+        OOR_LOG(LDBG_3, "tun_rm_fwd_from_entry: Removing all the forwarding entries association with the PeTRs for IPv4 EIDs");
         pxtr_fwd_tpl_list = (glist_t *)shash_lookup(data->eid_to_dp_entries,FULL_IPv4_ADDRESS_SPACE);
         /* Remove all the entries associated with the PxTR */
 
@@ -895,7 +926,7 @@ tun_rm_fwd_from_entry(lisp_addr_t *eid_prefix, uint8_t is_local)
             tun_rm_fwd_from_entry(fi->associated_entry,is_local);
         }
     }else if(strcmp(eid_prefix_char,FULL_IPv6_ADDRESS_SPACE) == 0){ // Update of the PeTR list for IPv6 EIDs or RTR list
-        OOR_LOG(LDBG_3, "tun_rm_fwd_from_entry: Removing all the forwading entries association with the PeTRs for IPv6 EIDs");
+        OOR_LOG(LDBG_3, "tun_rm_fwd_from_entry: Removing all the forwarding entries association with the PeTRs for IPv6 EIDs");
         pxtr_fwd_tpl_list = (glist_t *)shash_lookup(data->eid_to_dp_entries,FULL_IPv6_ADDRESS_SPACE);
         /* Remove all the entries associated with the PxTR */
 
@@ -907,10 +938,10 @@ tun_rm_fwd_from_entry(lisp_addr_t *eid_prefix, uint8_t is_local)
             tun_rm_fwd_from_entry(fi->associated_entry,is_local);
         }
     }else{
-        OOR_LOG(LDBG_3, "tun_rm_fwd_from_entry: Removing all the forwading entries association with the EID %s",eid_prefix_char);
+        OOR_LOG(LDBG_3, "tun_rm_fwd_from_entry: Removing all the forwarding entries association with the EID %s",eid_prefix_char);
         fwd_tpl_list = (glist_t *)shash_lookup(data->eid_to_dp_entries,eid_prefix_char);
         if (!fwd_tpl_list){
-            OOR_LOG(LDBG_1, "tun_rm_fwd_from_entry: Entry %s not found in the shasht!",eid_prefix_char);
+            OOR_LOG(LDBG_2, "tun_rm_fwd_from_entry: Entry %s not found in the shasht!",eid_prefix_char);
             return (BAD);
         }
         /* Check if it is a negative entry in order to remove also from PxTRs list */
