@@ -156,6 +156,8 @@ tun_output_unicast(lbuf_t *b, packet_tuple_t *tuple)
     fwd_entry_tuple_t *fe;
     glist_t *fwd_tuple_lst, *pxtr_fwd_tuple_list;
     tun_dplane_data_t *dp_data;
+    /* For xTR tuple->iid is 0 when received while for RTRs tuple->iid is the correct value */
+    uint32_t iid = tuple->iid;
 
     dp_data = tun_get_datap_data();
 
@@ -166,12 +168,16 @@ tun_output_unicast(lbuf_t *b, packet_tuple_t *tuple)
             return (BAD);
         }
         fe = (fwd_entry_tuple_t *)fi->dp_conf_inf;
-        if (fe && fe->srloc && fe->drloc)  {
+        if (!fe){
+            return (BAD);
+        }
+        if (fe->srloc && fe->drloc)  {
             fe->out_sock = get_out_socket_ptr_from_address(fe->srloc);
         }
-        // While we can not get iid from interface, we insert the tupla with iid = 0.
-        //   We only support a same EID prefix per xTR
-        fe->tuple->iid = 0;
+        // While we can not get iid from interface (xTR), we insert the tupla with iid = 0.
+        // For RTRs iid is initialized with the right value. Used to search in the table
+        // We only support a same EID prefix per xTR
+        fe->tuple->iid = iid;
         // fe->tuple is cloned from tuple
         if (ttable_insert(&(dp_data->ttable), fe->tuple, fi) != GOOD){
             /* If table is full, reset the data plane */
@@ -211,7 +217,7 @@ tun_output_unicast(lbuf_t *b, packet_tuple_t *tuple)
 
     /* Packets with no/negative map cache entry AND no PETR
      * OR packets with missing src or dst RLOCs*/
-    if (!fe || !fe->srloc || !fe->drloc) {
+    if (!fe->srloc || !fe->drloc) {
         switch (fi->neg_map_reply_act){
         case ACT_NO_ACTION:
         case ACT_SEND_MREQ:
@@ -229,10 +235,11 @@ tun_output_unicast(lbuf_t *b, packet_tuple_t *tuple)
 
     switch (fi->encap){
     case ENCP_LISP:
-        lisp_data_encap(b, LISP_DATA_PORT, LISP_DATA_PORT, fe->srloc, fe->drloc, fe->iid);
+        lisp_data_encap(b, fe->src_port, fe->dst_port, fe->srloc, fe->drloc, fe->iid);
         break;
     case ENCP_VXLAN_GPE:
-        vxlan_gpe_data_encap(b, VXLAN_GPE_DATA_PORT, VXLAN_GPE_DATA_PORT, fe->srloc, fe->drloc, fe->iid);
+        vxlan_gpe_data_encap(b, VXLAN_GPE_DATA_PORT, VXLAN_GPE_DATA_PORT, fe->srloc, fe->drloc, fe->iid,
+                             &tuple->dst_addr);
         break;
     }
 
