@@ -243,6 +243,11 @@ vpnapi_updated_addr(iface_t *iface,lisp_addr_t *old_addr,lisp_addr_t *new_addr)
 
     data = (vpnapi_data_t *)dplane_vpnapi.datap_data;
 
+    /* The address has been removed */
+    if (lisp_addr_lafi(new_addr) == LM_AFI_NO_ADDR){
+       return(GOOD);
+    }
+
     new_addr_ip_afi = lisp_addr_ip_afi(new_addr);
 
     /* Check if the detected change of address id the same. */
@@ -357,7 +362,7 @@ vpnapi_rm_fwd_from_entry(lisp_addr_t *eid_prefix, uint8_t is_local)
     }
 
     if (strcmp(eid_prefix_char,FULL_IPv4_ADDRESS_SPACE) == 0){ // Update of the PeTR list for IPv4 EIDs or RTR list
-        OOR_LOG(LDBG_3, "vpnapi_rm_fwd_from_entry: Removing all the forwading entries association with the PeTRs for IPv4 EIDs");
+        OOR_LOG(LDBG_3, "vpnapi_rm_fwd_from_entry: Removing all the forwarding entries association with the PeTRs for IPv4 EIDs");
         pxtr_fwd_tpl_list = (glist_t *)shash_lookup(data->eid_to_dp_entries,FULL_IPv4_ADDRESS_SPACE);
         /* Remove all the entries associated with the PxTR */
 
@@ -369,7 +374,7 @@ vpnapi_rm_fwd_from_entry(lisp_addr_t *eid_prefix, uint8_t is_local)
             vpnapi_rm_fwd_from_entry(fi->associated_entry,is_local);
         }
     }else if(strcmp(eid_prefix_char,FULL_IPv6_ADDRESS_SPACE) == 0){ // Update of the PeTR list for IPv6 EIDs or RTR list
-        OOR_LOG(LDBG_3, "vpnapi_rm_fwd_from_entry: Removing all the forwading entries association with the PeTRs for IPv6 EIDs");
+        OOR_LOG(LDBG_3, "vpnapi_rm_fwd_from_entry: Removing all the forwarding entries association with the PeTRs for IPv6 EIDs");
         pxtr_fwd_tpl_list = (glist_t *)shash_lookup(data->eid_to_dp_entries,FULL_IPv6_ADDRESS_SPACE);
         /* Remove all the entries associated with the PxTR */
 
@@ -381,10 +386,10 @@ vpnapi_rm_fwd_from_entry(lisp_addr_t *eid_prefix, uint8_t is_local)
             vpnapi_rm_fwd_from_entry(fi->associated_entry,is_local);
         }
     }else{
-        OOR_LOG(LDBG_3, "vpnapi_rm_fwd_from_entry: Removing all the forwading entries association with the EID %s",eid_prefix_char);
+        OOR_LOG(LDBG_3, "vpnapi_rm_fwd_from_entry: Removing all the forwarding entries association with the EID %s",eid_prefix_char);
         fwd_tpl_list = (glist_t *)shash_lookup(data->eid_to_dp_entries,eid_prefix_char);
         if (!fwd_tpl_list){
-            OOR_LOG(LDBG_1, "vpnapi_rm_fwd_from_entry: Entry %s not found in the shasht!",eid_prefix_char);
+            OOR_LOG(LDBG_2, "vpnapi_rm_fwd_from_entry: Entry %s not found in the shasht!",eid_prefix_char);
             return (BAD);
         }
         /* Check if it is a negative entry in order to remove also from PxTRs list */
