@@ -310,12 +310,24 @@ open_data_raw_input_socket(int afi, uint16_t port)
 int
 open_data_datagram_input_socket(int afi, int port)
 {
-
     int sock = ERR_SOCKET;
 
     if ((sock = open_udp_datagram_socket(afi)) < 0){
         return(ERR_SOCKET);
     }
+
+#ifdef  UDP_NO_CHECK6_RX
+    const int on = 1;
+    /* Disable IPv6 checksum computation for IPv6 data sockets (RFC 6935
+     * allows it) for tunneling protocols over IPv6 */
+    if (afi == AF_INET6) {
+        if (setsockopt(sock, IPPROTO_UDP, UDP_NO_CHECK6_RX, &on, sizeof(on)) < 0) {
+            OOR_LOG(LWRN, "open_data_datagram_input_socket: setsockopt UDP_NO_CHECK6_RX: %s",
+                    strerror(errno));
+        }
+    }
+#endif
+
     if(bind_socket(sock,afi,NULL,port) != GOOD){
         close(sock);
         return(ERR_SOCKET);
@@ -496,6 +508,14 @@ uconn_init(uconn_t *uc, int lp, int rp, lisp_addr_t *la,lisp_addr_t *ra)
     return(GOOD);
 }
 
+uconn_t *
+uconn_clone(uconn_t *uc)
+{
+    uconn_t *new_uc = xmalloc(sizeof(uconn_t));
+    memcpy(new_uc,uc,sizeof(uconn_t));
+    return (new_uc);
+}
+
 void
 uconn_from_5_tuple (packet_tuple_t *tuple, uconn_t *udp_con, uint8_t is_pkt_rx)
 {
