@@ -395,6 +395,10 @@ lisp_msg_put_mapping(
 
     /* Add locators */
     mapping_foreach_active_locator(m,loct){
+        // mapping_foreach_active_locator returns locators with status down
+        if (loct->state == DOWN){
+            continue;
+        }
         ploc = lisp_msg_put_locator(b, loct);
         if (probed_loc)
             if (probed_loc != NULL
