@@ -129,10 +129,7 @@ mapping_cmp(mapping_t *m1, mapping_t *m2)
     return (0);
 }
 
-/* Clones a mapping_t data structure
- * NOTE: it does not clone the 'extended_info'! This should be done by the
- * caller and in the future it shouldn't be done at all. 'extended_info'
- * should be moved out */
+/* Clones a mapping_t data structure */
 //XXX IT IS NOT CLONING LOCATORS
 mapping_t *
 mapping_clone(mapping_t *m) {
@@ -142,6 +139,7 @@ mapping_clone(mapping_t *m) {
     cm->authoritative = m->authoritative;
     cm->locator_count = m->locator_count;
     cm->ttl = m->ttl;
+    cm->locator_count = 0;
 
     return(cm);
 }
@@ -150,7 +148,7 @@ char *
 mapping_to_char(mapping_t *m)
 {
     locator_t *locator = NULL;
-    static char buf[500];
+    static char buf[1000];
     size_t buf_size = sizeof(buf);
 
 
@@ -228,7 +226,8 @@ mapping_add_locator(
 	return (result);
 }
 
-/* This function extract the locator from the list of locators of the mapping */
+/* This function extract the locator from the list of locators of the mapping.
+ * The locator is not destroyed*/
 int
 mapping_remove_locator(
         mapping_t *mapping,
@@ -257,11 +256,7 @@ mapping_remove_locator(
     }
 
     if (!lisp_addr_is_no_addr(addr)){
-        mapping->locator_count = mapping->locator_count - 1;
-    }
-
-    if (lisp_addr_is_no_addr(addr) == FALSE){
-        mapping->locator_count++;
+        mapping->locator_count--;
     }
 
     OOR_LOG(LDBG_2, "mapping_remove_locator: Removed locator %s from the mapping with"
@@ -427,10 +422,8 @@ mapping_sort_locators(mapping_t *mapping, lisp_addr_t *changed_loc_addr)
 }
 
 /*
- * Remove the locator from the non active locators list and reinsert in the correct list
- * The address of the locator should be modified before calling this function
- * This function is only used when an interface is down during the initial configuration
- * process and then is activated
+ * Remove the locator from the non active locators list and reinsert in the correct list.
+ * Locator should be non active locator. *
  */
 
 int
@@ -441,12 +434,6 @@ mapping_activate_locator(
 {
     int res = GOOD;
 
-    glist_t *loct_list = NULL;
-    loct_list = mapping_get_loct_lst_with_afi(mapping,LM_AFI_NO_ADDR,0);
-    if (loct_list == NULL){
-        return (BAD);
-    }
-
     mapping_remove_locator(mapping, loct);
 
     locator_clone_addr(loct,new_addr);
@@ -460,8 +447,72 @@ mapping_activate_locator(
     }else{
         locator_del(loct);
         OOR_LOG(LDBG_1,"mapping_activate_locator: Error activating the locator %s of the mapping %s. Locator couldn't be reinserted",
-                        lisp_addr_to_char(locator_addr(loct)),
-                        lisp_addr_to_char(&(mapping->eid_prefix)));
+                lisp_addr_to_char(locator_addr(loct)),
+                lisp_addr_to_char(&(mapping->eid_prefix)));
     }
     return (res);
 }
+
+/*
+ * Remove the locator from the active locators list and reinsert in the non active locator list
+ */
+
+int
+mapping_desactivate_locator(
+        mapping_t *mapping,
+        locator_t *loct)
+{
+    int res = GOOD;
+    lisp_addr_t *old_addr;
+
+    mapping_remove_locator(mapping, loct);
+
+    old_addr = locator_addr(loct); // We destroy later of printing msg
+    locator_set_addr(loct,lisp_addr_new_lafi(LM_AFI_NO_ADDR));
+    res = mapping_add_locator(mapping,loct);
+
+    if (res == GOOD){
+        OOR_LOG(LDBG_1,"mapping_desactivate_locator: The locator %s of the mapping %s has been desactivated",
+                lisp_addr_to_char(old_addr),
+                lisp_addr_to_char(&(mapping->eid_prefix)));
+        OOR_LOG(LDBG_2,"mapping_desactivate_locator: Updated mapping -> %s",mapping_to_char(mapping));
+    }else{
+        locator_del(loct);
+        OOR_LOG(LDBG_1,"mapping_desactivate_locator: Error deactivating the locator %s of the mapping %s. Locator couldn't be reinserted",
+                lisp_addr_to_char(old_addr),
+                lisp_addr_to_char(&(mapping->eid_prefix)));
+    }
+    lisp_addr_del(old_addr);
+    return (res);
+}
+
+// XXX This function is only used while we don't have support of L bit of ELPs
+int
+mapping_has_elp_with_l_bit(mapping_t *map)
+{
+    glist_t *loct_list;
+    locator_t *loct;
+    lisp_addr_t *addr;
+    elp_t * elp;
+    elp_node_t *elp_node;
+    glist_entry_t *loct_it;
+    glist_entry_t *elp_n_it;
+
+    loct_list = mapping_get_loct_lst_with_afi(map,LM_AFI_LCAF,LCAF_EXPL_LOC_PATH);
+    if (loct_list == NULL){
+        return (FALSE);
+    }
+    glist_for_each_entry(loct_it,loct_list){
+        loct = (locator_t *)glist_entry_data(loct_it);
+        addr = locator_addr(loct);
+        elp = (elp_t *)lisp_addr_lcaf_addr(addr);
+        glist_for_each_entry(elp_n_it,elp->nodes){
+            elp_node = (elp_node_t *)glist_entry_data(elp_n_it);
+            if (elp_node->L == true){
+                return (TRUE);
+            }
+        }
+    }
+
+    return (FALSE);
+}
