@@ -24,6 +24,7 @@
 #include <sys/prctl.h>
 #include <sys/stat.h>
 #include <time.h>
+#include <unistd.h>
 
 #include "oor.h"
 #if !defined(ANDROID) && !defined(OPENWRT)
@@ -41,9 +42,9 @@
 #include "control/lisp_ms.h"
 #include "data-plane/data-plane.h"
 #include "net_mgr/net_mgr.h"
+#include "lib/htable_ptrs.h"
 #include "lib/oor_log.h"
 #include "lib/nonces_table.h"
-#include "lib/pointers_table.h"
 #include "lib/sockets.h"
 #include "lib/timers.h"
 #include "lib/routing_tables_lib.h"
@@ -93,77 +94,9 @@ int oor_running;
 oor_api_connection_t oor_api_connection;
 #endif
 
-htable_nonces_t *nonces_ht;
-htable_ptrs_t *ptrs_to_timers_ht;
+htable_nonces_t *nonces_ht; //<uint64_t, oor_timer_t>
+htable_ptrs_t *ptrs_to_timers_ht; //<pointer, glist_t of timers>
 
-/**************************** FUNCTION DECLARATION ***************************/
-/* Check if oor is already running: /var/run/oor.pid */
-int pid_file_check_not_exist();
-
-/* Creates the PID file of the process */
-int pid_file_create();
-
-/* Remove the PID file of the process */
-void pid_file_remove();
-/*****************************************************************************/
-
-
-/* Check if oor is already running: /var/run/oor.pid */
-int
-pid_file_check_not_exist()
-{
-    FILE *pid_file;
-
-    pid_file = fopen(PID_FILE, "r");
-    if (pid_file != NULL)
-    {
-        OOR_LOG(LCRIT, "Check no other instance of oor is running. If no instance is running, remove %s",PID_FILE);
-        fclose(pid_file);
-        return (BAD);
-    }
-
-    return (GOOD);
-}
-
-/* Creates the PID file of the process */
-int
-pid_file_create()
-{
-    FILE *pid_file;
-    int pid = getpid();
-
-    pid_file = fopen(PID_FILE , "w");
-    if (pid_file == NULL){
-        OOR_LOG(LCRIT, "pid_file_create: Error creating PID file: %s",strerror(errno));
-        return (BAD);
-    }
-    fprintf(pid_file, "%d\n",pid);
-    fclose(pid_file);
-
-    OOR_LOG(LDBG_1, "PID file created: %s -> %d",PID_FILE, pid);
-
-    return (GOOD);
-}
-
-/* Remove the PID file of the process */
-void
-pid_file_remove()
-{
-    FILE *pid_file;
-
-    pid_file = fopen(PID_FILE, "r");
-    if (pid_file == NULL){
-        return;
-    }
-
-    fclose(pid_file);
-
-    if (remove(PID_FILE) != 0){
-        OOR_LOG(LWRN,"pid_file_remove: PID file couldn't be removed: %s", PID_FILE);
-    }else{
-        OOR_LOG(LDBG_1,"PID file removed");
-    }
-}
 
 /*
  *  Check for superuser privileges
@@ -246,9 +179,6 @@ void
 exit_cleanup(void) {
     OOR_LOG(LDBG_2,"Exit Cleanup");
 
-#ifndef ANDROID
-    pid_file_remove();
-#endif
     // Order is important
     ctrl_destroy(lctrl);
 
@@ -294,12 +224,22 @@ handle_oor_command_line(int argc, char **argv)
         exit_cleanup();
     }
 
-    if (args_info.daemonize_given) {
-        daemonize = TRUE;
-    }
     if (args_info.config_file_given) {
         config_file = strdup(args_info.config_file_arg);
     }
+
+    if (args_info.daemonize_given) {
+        if (config_file){
+            if (config_file[0] != '/'){
+                OOR_LOG(LCRIT, "Couldn't find config file %s. If you are useing OOR in daemon mode, please indicate a full path file.", config_file);
+            }
+            if (access( config_file, F_OK ) == -1 ) {
+                OOR_LOG(LCRIT, "Couldn't find config file %s.", config_file);
+            }
+        }
+        daemonize = TRUE;
+    }
+
     if (args_info.debug_given) {
         debug_level = args_info.debug_arg;
     }
@@ -411,10 +351,6 @@ initial_setup()
     if (check_capabilities() != GOOD){
         return (BAD);
     }
-    if(pid_file_check_not_exist() == BAD){
-        return (BAD);
-    }
-    pid_file_create();
 #endif
 
     /* Initialize the random number generator  */
@@ -439,7 +375,7 @@ int
 main(int argc, char **argv)
 {
     oor_dev_type_e dev_type;
-    lisp_xtr_t *tunnel_router;
+    tr_abstract_device *tunnel_router;
 
     if (initial_setup() != GOOD){
         exit(EXIT_SUCCESS);
@@ -477,8 +413,8 @@ main(int argc, char **argv)
 #endif
     if (dev_type == xTR_MODE || dev_type == RTR_MODE || dev_type == MN_MODE) {
         OOR_LOG(LDBG_2, "Configuring data plane");
-        tunnel_router = CONTAINER_OF(ctrl_dev, lisp_xtr_t, super);
-        if (data_plane->datap_init(dev_type,tr_get_encap_type(tunnel_router))!=GOOD){
+        tunnel_router = lisp_tr_abstract_cast(ctrl_dev);
+        if (data_plane->datap_init(dev_type,tr_encap_type(&tunnel_router->tr))!=GOOD){
             data_plane = NULL;
             exit_cleanup();
         }
@@ -530,14 +466,13 @@ JNIEXPORT jint JNICALL Java_org_openoverlayrouter_noroot_OOR_1JNI_oor_1start
     pid_t sid = 0;
     char log_file[1024];
     const char *path = NULL;
-    lisp_xtr_t *tunnel_router;
+    tr_abstract_device *tunnel_router;
     memset (log_file,0,sizeof(char)*1024);
     initial_setup();
     jni_init(env,thisObj);
     /* create socket master, timer wheel, initialize interfaces */
     smaster = sockmstr_create();
     oor_timers_init();
-
     /* create control. Only one instance for now */
     lctrl = ctrl_create();
 
@@ -560,6 +495,7 @@ JNIEXPORT jint JNICALL Java_org_openoverlayrouter_noroot_OOR_1JNI_oor_1start
     strcat(log_file,"oor.log");
     (*env)->ReleaseStringUTFChars(env, storage_path, path);
     open_log_file(log_file);
+    OOR_LOG(LDBG_2, "Default selected afi: %d",default_rloc_afi);
     if (parse_config_file()!=GOOD){
         exit_cleanup();
         close(vpn_tun_fd);
@@ -568,8 +504,8 @@ JNIEXPORT jint JNICALL Java_org_openoverlayrouter_noroot_OOR_1JNI_oor_1start
     dev_type = ctrl_dev_mode(ctrl_dev);
     if (dev_type == xTR_MODE || dev_type == RTR_MODE || dev_type == MN_MODE) {
         OOR_LOG(LDBG_2, "Configuring data plane");
-        tunnel_router = CONTAINER_OF(ctrl_dev, lisp_xtr_t, super);
-        if (data_plane->datap_init(dev_type, tr_get_encap_type(tunnel_router), vpn_tun_fd) != GOOD){
+        tunnel_router = lisp_tr_abstract_cast(ctrl_dev);
+        if (data_plane->datap_init(dev_type, tr_encap_type(&tunnel_router->tr), vpn_tun_fd) != GOOD){
             data_plane = NULL;
             return (BAD);
         }
@@ -580,7 +516,6 @@ JNIEXPORT jint JNICALL Java_org_openoverlayrouter_noroot_OOR_1JNI_oor_1start
     if (net_mgr->netm_init() != GOOD){
         exit_cleanup();
     }
-
      return (GOOD);
 }
 
